
package cz.upol.pja.lecture04.controllers;

import cz.upol.pja.lecture04.model.Customer;
import cz.upol.pja.lecture04.model.Invoice;
import cz.upol.pja.lecture04.model.InvoiceItem;
import cz.upol.pja.lecture04.model.StockItem;
import java.io.Serializable;
import java.math.BigDecimal;
import java.util.List;
import javax.inject.Named;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.util.Date;
import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.convert.Converter;
import javax.faces.view.ViewScoped;

@ViewScoped
@Named("invoicesCtl")
public class InvoicesCtl implements Serializable {
    
    private final EntityManagerFactory emfactory = Persistence.createEntityManagerFactory("lecture04PU");
    private final EntityManager entityManager = emfactory.createEntityManager();
    
    private Invoice invoice = new Invoice();
    
    private BigDecimal amount = new BigDecimal(1);
    private StockItem item;
    

    public InvoicesCtl() {
        invoice.setDate(new Date());
    }
    
    public List<Invoice> getInvoices() {
        return (List<Invoice>) entityManager.createQuery("select item from Invoice item").getResultList();
    }
    
    public List<Customer> getCustomers() {
        return (List<Customer>) entityManager.createQuery("select c from Customer c").getResultList();
    }
    
    public List<StockItem> getStockItems() {
        return (List<StockItem>) entityManager.createQuery("select si from StockItem si").getResultList();
    }
    
    public void addItem() {
        invoice.addItem(new InvoiceItem(invoice, item, amount));
    }
    
    public String saveForm() {
        entityManager.getTransaction().begin();
        entityManager.persist(invoice);
        entityManager.getTransaction().commit();
        return "invoices";
    }
    
    /**
     * Konvertor prevadi objekty tridy Customer na retezec
     * odpovidajici atributu itin (a opacne)
     */
    public Converter<Customer> getCustomerConverter() {
        return new Converter<Customer>() {
            @Override
            public Customer getAsObject(FacesContext context, UIComponent component, String value) {
                for (Customer customer : getCustomers()) {
                    if (customer.getItin().equals(value)) return customer;
                }
                return null;
            }

            @Override
            public String getAsString(FacesContext context, UIComponent component, Customer value) {
                return value.getItin();
            }
        };
    }
    
    /**
     * Konvertor prevadi objekty tridy StockItem na retezec
     * odpovidajici nazvu polozky (a opacne)
     */
    public Converter<StockItem> getStockItemConverter() {
        return new Converter<StockItem>() {
            @Override
            public StockItem getAsObject(FacesContext context, UIComponent component, String value) {
                for (StockItem item : getStockItems()) {
                    if (item.getItemName().equals(value)) return item;
                }
                return null;
            }

            @Override
            public String getAsString(FacesContext context, UIComponent component, StockItem value) {
                return value.getItemName();
            }
        };
    }


    // <editor-fold defaultstate="collapsed" desc="getters & setters">
    public Invoice getInvoice() {
        return invoice;
    }

    public void setInvoice(Invoice invoice) {
        this.invoice = invoice;
    }

    public BigDecimal getAmount() {
        return amount;
    }

    public void setAmount(BigDecimal amount) {
        this.amount = amount;
    }

    public StockItem getItem() {
        return item;
    }

    public void setItem(StockItem item) {
        this.item = item;
    }
    // </editor-fold>
}
